/*
   	Tairo v0.61
    Copyright (C) 2004 Jonas Gehring
        
    Tairo is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    Tairo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tairo; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 30.10.04; 14:56:40

// Own headers
#include "tairo.h"						// Constants, proto-types etc.
#include "gfx.h"							// Sprite definitions
#include "logo.h"							// Logo data
#include "levels.h"						// The data for the levels
// Biult-in headers
#include <graph.h>						// Common (non-windowed) graphic routines
#include <kbd.h>							// Keyboard handling routines
#include <intr.h>							// Routines for creating interrupt handlers
#include <math.h>							// ANSI-compatible routines for floating point arithmetic
#include <peekpoke.h>					// Routines for easier access to the memory and to I/O ports
#include <stdio.h>						// ANSI-compatible file and TTY input/output routines
#include <stdlib.h>						// Some useful routines proposed by ANSI (sorting, searching, etc.)
#include <string.h>						// ANSI-compatible string manipulation routines
#include <system.h>						// System routines (for accessing system timers, queues, etc.)
// Other headers
#include "extgraph.h"					// Include ExtGraph v2.00beta4 by TI-Chess Team


START_OPTIONS options;

// Show startup-options
START_OPTIONS DoIntro(BOOL mode)
{
	BOOL tflag = FALSE;
	short i, key;
	short chs = 0, chst = 0;
	char msg_diff[20], msg_level[22];
	char lvl_name[19];
	
	ClearScr();
	
	// Default values
	if (!mode) {
		options.mode = 1;
		options.diff = 0;
		options.level = 0;
	}
	
	// Display the logo
	for (i = 0; i < 3; i++)
		GraySprite32_XOR(32+CALCX+(i*32), 5+CALCY, LOGOHEIGHT, GetLogo(LIGHT_PLANE, i), GetLogo(DARK_PLANE, i), GRAYPL);

	GrayDrawStrExt(0, 44+CALCY, "by saubue", A_SHADOWED | A_CENTERED | A_XOR, F_4x6);
	
	GraySprite8_XOR(5+CALCX, 57+CALCY+(chs*10), HEIGHT, GetSpr(LIGHT_PLANE, 0), GetSpr(DARK_PLANE, 0), GRAYPL);

	tmr = FALSE;

	do {
		// Display current settings	
		if (!savload.wininfo[options.level])
			sprintf(msg_level, "Level             %d%d ", (options.level+1)/10, (options.level+1)-((options.level+1)/10)*10);
		else
			sprintf(msg_level, "Level           %c %d%d ", 15, (options.level+1)/10, (options.level+1)-((options.level+1)/10)*10);
			
		sprintf(lvl_name, "(%s)", levelnames[options.level]);
		
		GrayDrawStrExt(0, 57+CALCY, msg_level, A_SHADOWED | A_CENTERED | A_REPLACE, F_6x8);		
		GrayDrawStrExt(49+CALCX, 59+CALCY, lvl_name, A_REPLACE, F_4x6);
		
		if (tflag) {
			GrayDrawRect(15+CALCX, 77+CALCY, 20+CALCX, 90+CALCY, COLOR_WHITE, RECT_FILLED);
			tflag = FALSE;
		}
		
		sprintf(msg_diff,  "Difficulty         %d ", options.diff);	
		
		if (!tmr) {
			GrayDrawStrExt(0, 67+CALCY, "Time             OFF ", A_SHADOWED | A_CENTERED | A_REPLACE, F_6x8);		
			GrayDrawStrExt2B(0, 77+CALCY, msg_diff, A_SHADOWED | A_CENTERED | A_XOR, F_6x8, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(LIGHT_PLANE));
		}
		else {
			GrayDrawStrExt(0, 67+CALCY, "Time              ON ", A_SHADOWED | A_CENTERED | A_REPLACE, F_6x8);
			GrayDrawStrExt(0, 77+CALCY, msg_diff, A_SHADOWED | A_CENTERED | A_REPLACE, F_6x8);		
		}
		
		// Wait for keypress
		key = Key_Wait();
		
		// Select option
		if (key == KEY_UP && chs > 0)
			chs--;
		if (key == KEY_DOWN) {
			if (chs == 1 && tmr)
				chs++;
			else if (!chs)
				chs++;
		}
		
		if (chs != chst) {
			GraySprite8_XOR(5+CALCX, 57+CALCY+(chst*10), HEIGHT, GetSpr(LIGHT_PLANE, 0), GetSpr(DARK_PLANE, 0), GRAYPL);
			GraySprite8_XOR(5+CALCX, 57+CALCY+(chs*10), HEIGHT, GetSpr(LIGHT_PLANE, 0), GetSpr(DARK_PLANE, 0), GRAYPL);
			chst = chs;
		}
			
		// Select Time (ON, OFF)
		if ((key == KEY_LEFT || key == KEY_RIGHT) && chs == 1) {
			GrayDrawRect(20+CALCX, 77+CALCY, 20+(20*6)+CALCX, 85+CALCY, COLOR_WHITE, RECT_FILLED);
			tmr = !tmr;
			options.mode = tmr+1;
			tflag = TRUE;
		}
		
		// Select Level
		if (key == KEY_LEFT && !chs) {
			if (!options.level)
				options.level = MAX_LEVELS-1;
			else
				options.level--;
		}
		if (key == KEY_RIGHT && !chs) {
			if (options.level == MAX_LEVELS-1)
				options.level = 0;
			else
				options.level++;
		}
		if (key == KEY_RIGHT2ND && !chs) {
			if (options.level+5 >= MAX_LEVELS-1)
				options.level = MAX_LEVELS-1;
			else
				options.level += 5;
		}
		if (key == KEY_LEFT2ND && !chs) {
			if (options.level-5 <= 0)
				options.level = 0;
			else
				options.level -= 05;
		}
		
		// Select Difficulty (0-5)
		if (key == KEY_LEFT && chs == 2) {
			if (!options.diff)
				options.diff = 5;
			else
				options.diff--;
		}
		if (key == KEY_RIGHT && chs == 2) {
			if (options.diff == 5)
				options.diff = 0;
			else
				options.diff++; 
		}
		
		if (key == KEY_ENTER)
			return options;
			
	} while (key != KEY_ESC);
	
	options.mode = 0;
	
	return options;
}


// Draws the playing field
void DrawField(short level)
{
	char lvlmsg[17];
	unsigned short i, width = 0;
	
	ClearScr();
	
	// Draw the borders etc.
	GrayDrawRect(5+CALCX, 5+CALCY, 94+CALCX, 94+CALCY, COLOR_BLACK, RECT_EMPTY);
	GrayDrawRect(6+CALCX, 6+CALCY, 93+CALCX, 93+CALCY, COLOR_DARKGRAY, RECT_EMPTY);
	GrayDrawRect(7+CALCX, 7+CALCY, 92+CALCX, 92+CALCY, COLOR_DARKGRAY, RECT_EMPTY);
	GrayDrawRect(8+CALCX, 8+CALCY, 91+CALCX, 91+CALCY, COLOR_LIGHTGRAY, RECT_EMPTY);
	
	// Set level name
	strcpy(lvlmsg, levelnames[level]); 
	
	FontSetSys(F_4x6);
	
	// Calculate width of a string)
	for (i = 0; i < strlen(lvlmsg); i++)
		width += (FontCharWidth(lvlmsg[i])/2); 
	
	GrayDrawStrExt(108+CALCX, 15+CALCY, "Coins left:", A_SHADOWED | A_XOR, F_4x6); 
	GrayDrawStrExt(108+CALCX, 45+CALCY, "Time left:", A_SHADOWED | A_XOR, F_4x6);
	GrayDrawStrExt(114+CALCX, 75+CALCY, "Level:", A_SHADOWED | A_XOR, F_4x6);
	GrayDrawStrExt(125-width+CALCX, 85+CALCY, lvlmsg, A_SHADOWED | A_XOR, F_4x6);
}


// Draw "cursor" to select a direction to jump
void DrawSel(short x, short y, short olddir, short dir)
{
	signed short x1 = 0, y1 = 0;
	
	if (!dir)						// To the right
		x1 = 16;
	else if (dir == 1) 	// To the left
		y1 = -16;
	else if (dir == 2)	// Upwards
		x1 = -16;
	else if (dir == 3)	// Downwards
		y1 = 16;
	
	FastDrawLine(GrayGetPlane(LIGHT_PLANE), 23+CALCX+(x*9), 23+CALCY+(y*9), 23+x1+CALCX+(x*9), 23+y1+CALCY+(y*9), A_XOR); 
	FastDrawLine(GrayGetPlane(DARK_PLANE), 23+CALCX+(x*9), 23+CALCY+(y*9), 23+x1+CALCX+(x*9), 23+y1+CALCY+(y*9), A_XOR);  
	
	// Delete old "cursor" if needed
	if (olddir == dir)
		return;
	
	x1 = 0; y1 = 0;
	
	if (!olddir)						// To the right
		x1 = 16;
	else if (olddir == 1) 	// To the left
		y1 = -16;
	else if (olddir == 2)		// Upwards
		x1 = -16;
	else if (olddir == 3)		// Downwards
		y1 = 16;
	
	FastDrawLine(GrayGetPlane(LIGHT_PLANE), 23+CALCX+(x*9), 23+CALCY+(y*9), 23+x1+CALCX+(x*9), 23+y1+CALCY+(y*9), A_XOR); 
	FastDrawLine(GrayGetPlane(DARK_PLANE), 23+CALCX+(x*9), 23+CALCY+(y*9), 23+x1+CALCX+(x*9), 23+y1+CALCY+(y*9), A_XOR);  	
}


// Makes sure that the user hasn't pressed ESC without wanting to quit
BOOL ReallyQuit(void)
{	
	OSVFreeTimer(1);
	OSVFreeTimer(2);
	DrawBox();

	GrayDrawStrExt(0, (LCD_HEIGHT/2)-3, "Quit current game?", A_CENTERED | A_SHADOWED | A_XOR, F_4x6);

	if (Key_Wait() == KEY_ENTER)
		return TRUE;
	else {
		OSVRegisterTimer(1, 10, CursorBlink);
		if (tmr) 
			OSVRegisterTimer(2, 20, StopTimer);
	}
	return FALSE;
}


// Fills the empty space on calcs with 240*128-screen with sprites
void FillWithBalls(void)
{
	short x,y;
		
	for (x = 0; x < 30; x++) {
		for (y = 0; y < 14; y++) {
			// Fill the space on the left and right side
			if (x < 5 && y > 1) {
				if (!((x+1) % 2)) {
					GrayTile8x8_XOR_R(x , y*8, GetSpr(LIGHT_PLANE, 0), GetSpr(DARK_PLANE, 0), GRAYPL);
					GrayTile8x8_XOR_R((29-x), y*8, GetSpr(LIGHT_PLANE, 1), GetSpr(DARK_PLANE, 1), GRAYPL);
				}	else {
					GrayTile8x8_XOR_R(x , y*8, GetSpr(LIGHT_PLANE, 1), GetSpr(DARK_PLANE, 1), GRAYPL);				
					GrayTile8x8_XOR_R((29-x) , y*8, GetSpr(LIGHT_PLANE, 0), GetSpr(DARK_PLANE, 0), GRAYPL);
				}
			// Fill the upper and lower space
			} else if (y < 2) {
				if (!((x+1) % 2)) {
					GrayTile8x8_XOR_R(x , y*8, GetSpr(LIGHT_PLANE, 0), GetSpr(DARK_PLANE, 0), GRAYPL);
					GrayTile8x8_XOR_R(x , (15-y)*8, GetSpr(LIGHT_PLANE, 0), GetSpr(DARK_PLANE, 0), GRAYPL);
				}	else {
					GrayTile8x8_XOR_R(x , y*8, GetSpr(LIGHT_PLANE, 1), GetSpr(DARK_PLANE, 1), GRAYPL);				
					GrayTile8x8_XOR_R(x , (15-y)*8, GetSpr(LIGHT_PLANE, 1), GetSpr(DARK_PLANE, 1), GRAYPL);
				}
			}
		}
	}
}


// Make the processor do something for a bit
void WaitABit(int delay)
{
	int temp, temp1;
	randomize();
	
	temp1 = 0;
	for (temp = 0; temp < delay; temp++)
		temp1 = random(delay);
}


// Animates a jump
void AnimateJump(short x0, short y0, short dir)
{
	unsigned char tempsp1[8], tempsp2[8];
	short i = 0, waitdelay = 550;
	short route[18] = {0,0,1,1,2,2,3,3,4,4,3,3,2,2,1,1,0,0};

	// 1=Right, 2=Up, 3=Left, 4=Down

	if (dir == 4)	// Jump downwards
	{
		for (i = 0; i < 18; i++)
		{
			// Get sprite
			Sprite8Get(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)+i, HEIGHT, GrayGetPlane(DARK_PLANE), tempsp1);
			Sprite8Get(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)+i, HEIGHT, GrayGetPlane(LIGHT_PLANE), tempsp2);
		
			// Draw the new coin
			GraySprite8_RPLC_R(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)+i, HEIGHT, GetSpr(LIGHT_PLANE, FILLED), GetSpr(DARK_PLANE, FILLED), GRAYPL);							
		
			WaitABit(waitdelay);
		
			// Draw the temporary sprite
			GraySprite8_RPLC_R(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)+i, HEIGHT, tempsp1, tempsp2, GRAYPL);			
		}
	}
	else if (dir == 2)	// Jump upwards
	{
		for (i = 0; i < 18; i++)
		{
			// Get sprite
			Sprite8Get(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)-i, HEIGHT, GrayGetPlane(DARK_PLANE), tempsp1);
			Sprite8Get(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)-i, HEIGHT, GrayGetPlane(LIGHT_PLANE), tempsp2);
		
			// Draw the new coin
			GraySprite8_RPLC_R(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)-i, HEIGHT, GetSpr(LIGHT_PLANE, FILLED), GetSpr(DARK_PLANE, FILLED), GRAYPL);							
		
			WaitABit(waitdelay);
		
			// Draw the temporary sprite
			GraySprite8_RPLC_R(19+CALCX+(x0*9)+route[i], 19+CALCY+(y0*9)-i, HEIGHT, tempsp1, tempsp2, GRAYPL);			
		}	
	}
	else if (dir == 1)	// Jump to the right
	{
		for (i = 0; i < 18; i++)
		{
			// Get sprite
			Sprite8Get(19+CALCX+(x0*9)+i, 19+CALCY+(y0*9)-route[i], HEIGHT, GrayGetPlane(DARK_PLANE), tempsp1);
			Sprite8Get(19+CALCX+(x0*9)+i, 19+CALCY+(y0*9)-route[i], HEIGHT, GrayGetPlane(LIGHT_PLANE), tempsp2);
		
			// Draw the new coin
			GraySprite8_RPLC_R(19+CALCX+(x0*9)+i, 19+CALCY+(y0*9)-route[i], HEIGHT, GetSpr(LIGHT_PLANE, FILLED), GetSpr(DARK_PLANE, FILLED), GRAYPL);							
		
			WaitABit(waitdelay);	
			
			// Draw the temporary sprite
			GraySprite8_RPLC_R(19+CALCX+(x0*9)+i, 19+CALCY+(y0*9)-route[i], HEIGHT, tempsp1, tempsp2, GRAYPL);			
		}	
	}
	else if (dir == 3)	// Jump to the left
	{
		for (i = 0; i < 18; i++)
		{
			// Get sprite
			Sprite8Get(19+CALCX+(x0*9)-i, 19+CALCY+(y0*9)-route[i], HEIGHT, GrayGetPlane(DARK_PLANE), tempsp1);
			Sprite8Get(19+CALCX+(x0*9)-i, 19+CALCY+(y0*9)-route[i], HEIGHT, GrayGetPlane(LIGHT_PLANE), tempsp2);
		
			// Draw the new coin
			GraySprite8_RPLC_R(19+CALCX+(x0*9)-i, 19+CALCY+(y0*9)-route[i], HEIGHT, GetSpr(LIGHT_PLANE, FILLED), GetSpr(DARK_PLANE, FILLED), GRAYPL);							
		
			WaitABit(waitdelay);
			
			// Draw the temporary sprite
			GraySprite8_RPLC_R(19+CALCX+(x0*9)-i, 19+CALCY+(y0*9)-route[i], HEIGHT, tempsp1, tempsp2, GRAYPL);			
		}	
	}
	//SetIntVec (AUTO_INT_5, OldInt5);
}